@@ -40,7 +40,7 @@ typedef struct
 	SSL_CTX *ctx;
 	unsigned int verify:1;
 } GIOSSLChannel;
-	
+
 static SSL_CTX *ssl_ctx = NULL;
 
 static void irssi_ssl_free(GIOChannel *handle)
@@ -93,38 +93,43 @@ static gboolean irssi_ssl_verify(SSL *ssl, SSL_CTX *ctx, X509 *cert)
 	return TRUE;
 }
 
-static GIOStatus ssl_errno(gint e)
-{
-	switch(e)
-	{
-		case EINVAL:
-			return G_IO_STATUS_ERROR;
-		case EINTR:
-		case EAGAIN:
-			return G_IO_STATUS_AGAIN;
-		default:
-			return G_IO_STATUS_ERROR;
-	}
-	/*UNREACH*/
-	return G_IO_STATUS_ERROR;
-}
-
 static GIOStatus irssi_ssl_read(GIOChannel *handle, gchar *buf, gsize len, gsize *ret, GError **gerr)
 {
 	GIOSSLChannel *chan = (GIOSSLChannel *)handle;
-	gint err;
-	
-	err = SSL_read(chan->ssl, buf, len);
-	if(err < 0)
+	gint ret1, err;
+	const char *errstr;
+
+	ret1 = SSL_read(chan->ssl, buf, len);
+	if(ret1 <= 0)
 	{
 		*ret = 0;
-		if(SSL_get_error(chan->ssl, err) == SSL_ERROR_WANT_READ)
+		err = SSL_get_error(chan->ssl, ret1);
+		if(err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE)
 			return G_IO_STATUS_AGAIN;
-		return ssl_errno(errno);
+		else if(err == SSL_ERROR_ZERO_RETURN)
+			return G_IO_STATUS_EOF;
+		else if (err == SSL_ERROR_SYSCALL)
+		{
+			errstr = ERR_reason_error_string(ERR_get_error());
+			if (errstr == NULL && ret1 == -1)
+				errstr = strerror(errno);
+			if (errstr == NULL)
+				errstr = "server closed connection unexpectedly";
+		}
+		else
+		{
+			errstr = ERR_reason_error_string(ERR_get_error());
+			if (errstr == NULL)
+				errstr = "unknown SSL error";
+		}
+		g_warning("SSL read error: %s", errstr);
+		*gerr = g_error_new_literal(G_IO_CHANNEL_ERROR, G_IO_CHANNEL_ERROR_FAILED,
+					    errstr);
+		return G_IO_STATUS_ERROR;
 	}
 	else
 	{
-		*ret = err;
+		*ret = ret1;
 		return G_IO_STATUS_NORMAL;
 	}
 	/*UNREACH*/
@@ -134,19 +139,40 @@ static GIOStatus irssi_ssl_read(GIOChannel *handle, gchar *buf, gsize len, gsize
 static GIOStatus irssi_ssl_write(GIOChannel *handle, const gchar *buf, gsize len, gsize *ret, GError **gerr)
 {
 	GIOSSLChannel *chan = (GIOSSLChannel *)handle;
-	gint err;
+	gint ret1, err;
+	const char *errstr;
 
-	err = SSL_write(chan->ssl, (const char *)buf, len);
-	if(err < 0)
+	ret1 = SSL_write(chan->ssl, (const char *)buf, len);
+	if(ret1 <= 0)
 	{
 		*ret = 0;
-		if(SSL_get_error(chan->ssl, err) == SSL_ERROR_WANT_READ)
+		err = SSL_get_error(chan->ssl, ret1);
+		if(err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE)
 			return G_IO_STATUS_AGAIN;
-		return ssl_errno(errno);
+		else if(err == SSL_ERROR_ZERO_RETURN)
+			errstr = "server closed connection";
+		else if (err == SSL_ERROR_SYSCALL)
+		{
+			errstr = ERR_reason_error_string(ERR_get_error());
+			if (errstr == NULL && ret1 == -1)
+				errstr = strerror(errno);
+			if (errstr == NULL)
+				errstr = "server closed connection unexpectedly";
+		}
+		else
+		{
+			errstr = ERR_reason_error_string(ERR_get_error());
+			if (errstr == NULL)
+				errstr = "unknown SSL error";
+		}
+		g_warning("SSL write error: %s", errstr);
+		*gerr = g_error_new_literal(G_IO_CHANNEL_ERROR, G_IO_CHANNEL_ERROR_FAILED,
+					    errstr);
+		return G_IO_STATUS_ERROR;
 	}
 	else
 	{
-		*ret = err;
+		*ret = ret1;
 		return G_IO_STATUS_NORMAL;
 	}
 	/*UNREACH*/
@@ -156,17 +182,15 @@ static GIOStatus irssi_ssl_write(GIOChannel *handle, const gchar *buf, gsize len
 static GIOStatus irssi_ssl_seek(GIOChannel *handle, gint64 offset, GSeekType type, GError **gerr)
 {
 	GIOSSLChannel *chan = (GIOSSLChannel *)handle;
-	GIOError e;
-	e = g_io_channel_seek(chan->giochan, offset, type);
-	return (e == G_IO_ERROR_NONE) ? G_IO_STATUS_NORMAL : G_IO_STATUS_ERROR;
+
+	return chan->giochan->funcs->io_seek(handle, offset, type, gerr);
 }
 
 static GIOStatus irssi_ssl_close(GIOChannel *handle, GError **gerr)
 {
 	GIOSSLChannel *chan = (GIOSSLChannel *)handle;
-	g_io_channel_close(chan->giochan);
 
-	return G_IO_STATUS_NORMAL;
+	return chan->giochan->funcs->io_close(handle, gerr);
 }
 
 static GSource *irssi_ssl_create_watch(GIOChannel *handle, GIOCondition cond)
@@ -205,7 +229,7 @@ static gboolean irssi_ssl_init(void)
 {
 	SSL_library_init();
 	SSL_load_error_strings();
-	
+
 	ssl_ctx = SSL_CTX_new(SSLv23_client_method());
 	if(!ssl_ctx)
 	{
@@ -221,19 +245,19 @@ static GIOChannel *irssi_ssl_get_iochannel(GIOChannel *handle, const char *mycer
 {
 	GIOSSLChannel *chan;
 	GIOChannel *gchan;
-	int err, fd;
+	int fd;
 	SSL *ssl;
 	SSL_CTX *ctx = NULL;
 
 	g_return_val_if_fail(handle != NULL, NULL);
-	
+
 	if(!ssl_ctx && !irssi_ssl_init())
 		return NULL;
 
 	if(!(fd = g_io_channel_unix_get_fd(handle)))
 		return NULL;
 
-	if (mycert && *mycert) {	
+	if (mycert && *mycert) {
 		char *scert = NULL, *spkey = NULL;
 		if ((ctx = SSL_CTX_new(SSLv23_client_method())) == NULL) {
 			g_error("Could not allocate memory for SSL context");
@@ -277,14 +301,14 @@ static GIOChannel *irssi_ssl_get_iochannel(GIOChannel *handle, const char *mycer
 
 	if (ctx == NULL)
 		ctx = ssl_ctx;
-	
+
 	if(!(ssl = SSL_new(ctx)))
 	{
 		g_warning("Failed to allocate SSL structure");
 		return NULL;
 	}
 
-	if(!(err = SSL_set_fd(ssl, fd)))
+	if(!SSL_set_fd(ssl, fd))
 	{
 		g_warning("Failed to associate socket to SSL stream");
 		SSL_free(ssl);
@@ -303,7 +327,9 @@ static GIOChannel *irssi_ssl_get_iochannel(GIOChannel *handle, const char *mycer
 	gchan = (GIOChannel *)chan;
 	gchan->funcs = &irssi_ssl_channel_funcs;
 	g_io_channel_init(gchan);
-	
+	gchan->is_readable = gchan->is_writeable = TRUE;
+	gchan->use_buffer = FALSE;
+
 	return gchan;
 }
 
@@ -330,12 +356,25 @@ int irssi_ssl_handshake(GIOChannel *handle)
 	ret = SSL_connect(chan->ssl);
 	if (ret <= 0) {
 		err = SSL_get_error(chan->ssl, ret);
-		if (err != SSL_ERROR_WANT_READ && err != SSL_ERROR_WANT_WRITE) {
-			errstr = ERR_reason_error_string(ERR_get_error());
-			g_warning("SSL handshake failed: %s", errstr != NULL ? errstr : "server closed connection");
-			return -1;
+		switch (err) {
+			case SSL_ERROR_WANT_READ:
+				return 1;
+			case SSL_ERROR_WANT_WRITE:
+				return 3;
+			case SSL_ERROR_ZERO_RETURN:
+				g_warning("SSL handshake failed: %s", "server closed connection");
+				return -1;
+			case SSL_ERROR_SYSCALL:
+				errstr = ERR_reason_error_string(ERR_get_error());
+				if (errstr == NULL && ret == -1)
+					errstr = strerror(errno);
+				g_warning("SSL handshake failed: %s", errstr != NULL ? errstr : "server closed connection unexpectedly");
+				return -1;
+			default:
+				errstr = ERR_reason_error_string(ERR_get_error());
+				g_warning("SSL handshake failed: %s", errstr != NULL ? errstr : "unknown SSL error");
+				return -1;
 		}
-		return err == SSL_ERROR_WANT_READ ? 1 : 3;
 	}
 
 	cert = SSL_get_peer_certificate(chan->ssl);
