@@ -21,7 +21,7 @@
 #include "module.h"
 #include "signals.h"
 #include "commands.h"
-#include "line-split.h"
+#include "net-disconnect.h"
 #include "net-nonblock.h"
 #include "net-sendbuffer.h"
 #include "misc.h"
@@ -77,8 +77,8 @@ static char *server_create_address_tag(const char *address)
 	/* try to generate a reasonable server tag */
 	if (strchr(address, '.') == NULL) {
 		start = end = NULL;
-	} else if (g_strncasecmp(address, "irc", 3) == 0 ||
-	    g_strncasecmp(address, "chat", 4) == 0) {
+	} else if (g_ascii_strncasecmp(address, "irc", 3) == 0 ||
+	    g_ascii_strncasecmp(address, "chat", 4) == 0) {
 		/* irc-2.cs.hut.fi -> hut, chat.bt.net -> bt */
 		end = strrchr(address, '.');
 		start = end-1;
@@ -128,7 +128,7 @@ static char *server_create_tag(SERVER_CONNECT_REC *conn)
 	num = 2;
 	while (server_find_tag(str->str) != NULL ||
 	       server_find_lookup_tag(str->str) != NULL) {
-		g_string_sprintf(str, "%s%d", tag, num);
+		g_string_printf(str, "%s%d", tag, num);
 		num++;
 	}
 	g_free(tag);
@@ -535,7 +535,6 @@ int server_unref(SERVER_REC *server)
         MODULE_DATA_DEINIT(server);
 	server_connect_unref(server->connrec);
 	if (server->rawlog != NULL) rawlog_destroy(server->rawlog);
-	if (server->buffer != NULL) line_split_free(server->buffer);
 	g_free(server->version);
 	g_free(server->away_reason);
 	g_free(server->nick);
