@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -175,8 +175,8 @@ static void cmd_window_new(const char *data, void *server, WI_ITEM_REC *item)
 
 	g_return_if_fail(data != NULL);
 
-	type = (g_strncasecmp(data, "hid", 3) == 0 || g_strcasecmp(data, "tab") == 0) ? 1 :
-		(g_strcasecmp(data, "split") == 0 ? 2 : 0);
+	type = (g_ascii_strncasecmp(data, "hid", 3) == 0 || g_ascii_strcasecmp(data, "tab") == 0) ? 1 :
+		(g_ascii_strcasecmp(data, "split") == 0 ? 2 : 0);
 	signal_emit("gui window create override", 1, GINT_TO_POINTER(type));
 
 	window = window_create(NULL, FALSE);
@@ -276,6 +276,45 @@ static WINDOW_REC *window_highest_activity(WINDOW_REC *window)
 	return max_win;
 }
 
+static inline int is_nearer(int r1, int r2)
+{
+	int a = r2 < active_win->refnum;
+	int b = r1 < r2;
+
+	if (r1 > active_win->refnum)
+		return a || b;
+	else
+		return a && b;
+}
+
+static WINDOW_REC *window_find_item_cycle(SERVER_REC *server, const char *name)
+{
+	WINDOW_REC *rec, *win;
+	GSList *tmp;
+
+	win = NULL;
+
+	tmp = g_slist_find(windows, active_win);
+	tmp = tmp->next;
+	for (;; tmp = tmp->next) {
+		if (tmp == NULL)
+			tmp = windows;
+
+		if (tmp->data == active_win)
+			break;
+
+		rec = tmp->data;
+
+		if (window_item_find_window(rec, server, name) != NULL &&
+		    (win == NULL || is_nearer(rec->refnum, win->refnum))) {
+			win = rec;
+			if (server != NULL) break;
+		}
+	}
+
+	return win;
+}
+
 /* SYNTAX: WINDOW GOTO active|<number>|<name> */
 static void cmd_window_goto(const char *data)
 {
@@ -293,10 +332,15 @@ static void cmd_window_goto(const char *data)
 	if (!cmd_get_params(data, &free_arg, 1, &target))
 		return;
 
-	if (g_strcasecmp(target, "active") == 0)
+	if (g_ascii_strcasecmp(target, "active") == 0)
                 window = window_highest_activity(active_win);
-	else
-                window = window_find_item(active_win->active_server, target);
+	else {
+		window = window_find_name(target);
+		if (window == NULL && active_win->active_server != NULL)
+			window = window_find_item_cycle(active_win->active_server, target);
+		if (window == NULL)
+			window = window_find_item_cycle(NULL, target);
+	}
 
 	if (window != NULL)
 		window_set_active(window);
@@ -356,11 +400,11 @@ static void cmd_window_immortal(const char *data)
 
 	if (*data == '\0')
 		set = active_win->immortal;
-	else if (g_strcasecmp(data, "ON") == 0)
+	else if (g_ascii_strcasecmp(data, "ON") == 0)
                 set = TRUE;
-	else if (g_strcasecmp(data, "OFF") == 0)
+	else if (g_ascii_strcasecmp(data, "OFF") == 0)
                 set = FALSE;
-	else if (g_strcasecmp(data, "TOGGLE") == 0)
+	else if (g_ascii_strcasecmp(data, "TOGGLE") == 0)
                 set = !active_win->immortal;
 	else {
 		printformat_window(active_win, MSGLEVEL_CLIENTERROR,
