@@ -3,35 +3,10 @@
 #define MODULE_NAME "proxy"
 
 #include "network.h"
-#include "line-split.h"
 #include "irc.h"
 #include "irc-servers.h"
 
-typedef struct {
-	int port;
-	char *ircnet;
-
-	int tag;
-	GIOChannel *handle;
-
-	GSList *clients;
-} LISTEN_REC;
-
-typedef struct {
-	LINEBUF_REC *buffer;
-
-	char *nick, *host;
-	GIOChannel *handle;
-	int tag;
-
-	char *proxy_address;
-	LISTEN_REC *listen;
-	IRC_SERVER_REC *server;
-	unsigned int pass_sent:1;
-	unsigned int user_sent:1;
-	unsigned int connected:1;
-	unsigned int want_ctcp:1;
-} CLIENT_REC;
+#include "proxy.h"
 
 extern GSList *proxy_listens;
 extern GSList *proxy_clients;
